﻿/******************************************************************************
 * JRC NRD-353(D)に対して予め決まっている周波数、モードなどの設定をする
 * 
 * １．想定：FT8、短波放送
 * ２．接続：NRD-535とWindows PCのRS232C端子間をリバースケーブルで接続
 * ３．主な機能
 * 　(1)プログラムで設定された各バンドのFT8周波数、モードの設定
 * 　(2)短波放送の局名、周波数、備考情報をユーザーが登録し、保存・読み出し
 * 
 * 2020年8月23日 Y.Muto/JF1RSB
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.IO;

namespace NRD_535_Commander
{
    public partial class Form1 : Form
    {
        // コマンド定数の設定（本プログラム使用していないコマンドはコメントアウト）

        // リモートON/OFFコマンド
        string rmtON = "H1";
        string rmtOFF = "H0";

        // FT8受信周波数変数の設定
        string frqFT8_160m = "F01840000";
        string frqFT8_80m = "F03573000";
        string frqFT8_40m = "F07074000";
        string frqFT8_30m = "F10136000";
        string frqFT8_20m = "F14074000";
        string frqFT8_17m = "F18100000";
        string frqFT8_15m = "F21074000";
        string frqFT8_12m = "F24915000";
        string frqFT8_10m = "F28074000";

        // モード設定コマンド
//        string modRTTY = "D0";
//        string modCW = "D1";
        string modUSB = "D2";
//        string modLSB = "D3";
        string modAM = "D4";
//        string modFM = "D5";
//        string modFAX = "D6";
//        string modECSS_USB = "D7";
//        string modECSS_LSB = "D8";

        // バンド幅設定コマンド
        string bwWIDE = "B0";
        string bwINTER = "B1";
        //        string bwNARR = "B2";
        //        string bwAUX = "B3";

        // AGCスピード設定コマンド
        string agcSlow = "G0";
        string agcFast = "G1";
 //       string agcOff = "G2";

        public Form1()
        {
            InitializeComponent();
            btnConnect.BackColor = SystemColors.Control;
            btnConnect.UseVisualStyleBackColor = true;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // UTC時計
            timer1.Interval = 1000;
            timer1.Enabled = true;

            // 利用可能なシリアルポート名を取得してコンボボックスにセットする
            string[] PortList = SerialPort.GetPortNames();
            cmbPortName.Items.Clear();
            foreach (string PortName in PortList)
            {
                cmbPortName.Items.Add(PortName);
            }
            if (cmbPortName.Items.Count > 0)
            {
                cmbPortName.SelectedIndex = 0;
            }

            // app.exe.configから設定値を取得する
            GetUserData();
        }

        // app.exe.configからのユーザー設定値取得メソッド
        public void GetUserData()
        {
            cmbPortName.Text = Properties.Settings.Default.ComNumber;  // シリアルポート
            // ラジオ放送プリセット
            textBoxCH1Name.Text = Properties.Settings.Default.CH1Name;
            textBoxCH2Name.Text = Properties.Settings.Default.CH2Name;
            textBoxCH3Name.Text = Properties.Settings.Default.CH3Name;
            textBoxCH4Name.Text = Properties.Settings.Default.CH4Name;
            textBoxCH5Name.Text = Properties.Settings.Default.CH5Name;
            textBoxCH6Name.Text = Properties.Settings.Default.CH6Name;
            textBoxCH7Name.Text = Properties.Settings.Default.CH7Name;
            textBoxCH8Name.Text = Properties.Settings.Default.CH8Name;
            textBoxCH9Name.Text = Properties.Settings.Default.CH9Name;
            textBoxCH10Name.Text = Properties.Settings.Default.CH10Name;
            textBoxCH11Name.Text = Properties.Settings.Default.CH11Name;
            textBoxCH12Name.Text = Properties.Settings.Default.CH12Name;
            textBoxCH13Name.Text = Properties.Settings.Default.CH13Name;

            textBoxCH1Frq.Text = Properties.Settings.Default.CH1Frq;
            textBoxCH2Frq.Text = Properties.Settings.Default.CH2Frq;
            textBoxCH3Frq.Text = Properties.Settings.Default.CH3Frq;
            textBoxCH4Frq.Text = Properties.Settings.Default.CH4Frq;
            textBoxCH5Frq.Text = Properties.Settings.Default.CH5Frq;
            textBoxCH6Frq.Text = Properties.Settings.Default.CH6Frq;
            textBoxCH7Frq.Text = Properties.Settings.Default.CH7Frq;
            textBoxCH8Frq.Text = Properties.Settings.Default.CH8Frq;
            textBoxCH9Frq.Text = Properties.Settings.Default.CH9Frq;
            textBoxCH10Frq.Text = Properties.Settings.Default.CH10Frq;
            textBoxCH11Frq.Text = Properties.Settings.Default.CH11Frq;
            textBoxCH12Frq.Text = Properties.Settings.Default.CH12Frq;
            textBoxCH13Frq.Text = Properties.Settings.Default.CH13Frq;

            textBoxCH1Memo.Text = Properties.Settings.Default.CH1Memo;
            textBoxCH2Memo.Text = Properties.Settings.Default.CH2Memo;
            textBoxCH3Memo.Text = Properties.Settings.Default.CH3Memo;
            textBoxCH4Memo.Text = Properties.Settings.Default.CH4Memo;
            textBoxCH5Memo.Text = Properties.Settings.Default.CH5Memo;
            textBoxCH6Memo.Text = Properties.Settings.Default.CH6Memo;
            textBoxCH7Memo.Text = Properties.Settings.Default.CH7Memo;
            textBoxCH8Memo.Text = Properties.Settings.Default.CH8Memo;
            textBoxCH9Memo.Text = Properties.Settings.Default.CH9Memo;
            textBoxCH10Memo.Text = Properties.Settings.Default.CH10Memo;
            textBoxCH11Memo.Text = Properties.Settings.Default.CH11Memo;
            textBoxCH12Memo.Text = Properties.Settings.Default.CH12Memo;
            textBoxCH13Memo.Text = Properties.Settings.Default.CH13Memo;

        }

        // シリアルポート接続メソッド
        private void ComConnect()
        {
            serialPort1.PortName = cmbPortName.SelectedItem.ToString();
            serialPort1.BaudRate = 4800;
            serialPort1.Parity = Parity.None;
            serialPort1.DataBits = 8;
            serialPort1.StopBits = StopBits.One;
            serialPort1.Handshake = Handshake.None;
            serialPort1.Open();
            btnConnect.BackColor = Color.Aqua;
        }

        // シリアルポート切断メソッド
        private void ComDisConnect()
        {
            if (serialPort1.IsOpen)
            {
                serialPort1.Write("I0" + "\r");
                serialPort1.Close();
                btnConnect.BackColor = SystemColors.Control;
                btnConnect.UseVisualStyleBackColor = true;
                labelConnectStatus.Text = "NRD-535未接続";
                labelConnectStatus.ForeColor = System.Drawing.Color.Black;
            }
        }

        // FT8受信バンドコマンドメソッド
        public void SetFT8Rcv(string FT8band)
        {
            if (serialPort1.IsOpen)
            {
                serialPort1.Write(rmtON + "\r");
                serialPort1.Write(FT8band + "\r");
                serialPort1.Write(modUSB + "\r");
                serialPort1.Write(bwINTER + "\r");
                serialPort1.Write(agcFast + "\r");
                serialPort1.Write(rmtOFF + "\r");
            }
        }

        // ラジオ放送受信コマンドメソッド
        public void SetRadioRcv(string frqRadio)
        {
            if (serialPort1.IsOpen)  // シリアルポート未接続時は何もしない
            {
                if (frqRadio == "") // データ無しの場合は何もしない
                {
                    return;
                }
                string frqData = Regex.Replace(frqRadio, @"[^0-9]", "");  // 数字のみを抜き出す     
                if (frqData == "")
                {
                    return;
                }
                while (frqData.Substring(0, 1) == "0")  // 一番左の0を削除する
                {
                    if ( frqData == "" )
                    {
                        return;
                    }
                    frqData = frqData.Remove(0, 1);
                    if (frqData == "")
                    {
                        return;
                    }
                }
                if (frqData.Length > 5)  // 文字数が6以上の場合は何もしない
                {
                    return;
                }
                while (frqData.Length < 5)  // frqDataの左側に0を追加
                {
                    frqData = "0" + frqData;
                }
                frqData = "F" + frqData + "000";  // 必要な文字を追加して周波数設定コマンドを作る

                serialPort1.Write(rmtON + "\r");
                serialPort1.Write(frqData + "\r");
                serialPort1.Write(modAM + "\r");
                serialPort1.Write(bwWIDE + "\r");
                serialPort1.Write(agcSlow + "\r");
                serialPort1.Write(rmtOFF + "\r");
            } 
        }

        // ユーザーデータ保存メソッド
        public void SaveUserData()
        {
            Properties.Settings.Default.ComNumber = cmbPortName.Text;  // シリアルポート
            // ラジオ放送プリセット
            Properties.Settings.Default.CH1Name = textBoxCH1Name.Text;
            Properties.Settings.Default.CH2Name = textBoxCH2Name.Text;
            Properties.Settings.Default.CH3Name = textBoxCH3Name.Text;
            Properties.Settings.Default.CH4Name = textBoxCH4Name.Text;
            Properties.Settings.Default.CH5Name = textBoxCH5Name.Text;
            Properties.Settings.Default.CH6Name = textBoxCH6Name.Text;
            Properties.Settings.Default.CH7Name = textBoxCH7Name.Text;
            Properties.Settings.Default.CH8Name = textBoxCH8Name.Text;
            Properties.Settings.Default.CH9Name = textBoxCH9Name.Text;
            Properties.Settings.Default.CH10Name = textBoxCH10Name.Text;
            Properties.Settings.Default.CH11Name = textBoxCH11Name.Text;
            Properties.Settings.Default.CH12Name = textBoxCH12Name.Text;
            Properties.Settings.Default.CH13Name = textBoxCH13Name.Text;

            Properties.Settings.Default.CH1Frq = textBoxCH1Frq.Text;
            Properties.Settings.Default.CH2Frq = textBoxCH2Frq.Text;
            Properties.Settings.Default.CH3Frq = textBoxCH3Frq.Text;
            Properties.Settings.Default.CH4Frq = textBoxCH4Frq.Text;
            Properties.Settings.Default.CH5Frq = textBoxCH5Frq.Text;
            Properties.Settings.Default.CH6Frq = textBoxCH6Frq.Text;
            Properties.Settings.Default.CH7Frq = textBoxCH7Frq.Text;
            Properties.Settings.Default.CH8Frq = textBoxCH8Frq.Text;
            Properties.Settings.Default.CH9Frq = textBoxCH9Frq.Text;
            Properties.Settings.Default.CH10Frq = textBoxCH10Frq.Text;
            Properties.Settings.Default.CH11Frq = textBoxCH11Frq.Text;
            Properties.Settings.Default.CH12Frq = textBoxCH12Frq.Text;
            Properties.Settings.Default.CH13Frq = textBoxCH13Frq.Text;

            Properties.Settings.Default.CH1Memo = textBoxCH1Memo.Text;
            Properties.Settings.Default.CH2Memo = textBoxCH2Memo.Text;
            Properties.Settings.Default.CH3Memo = textBoxCH3Memo.Text;
            Properties.Settings.Default.CH4Memo = textBoxCH4Memo.Text;
            Properties.Settings.Default.CH5Memo = textBoxCH5Memo.Text;
            Properties.Settings.Default.CH6Memo = textBoxCH6Memo.Text;
            Properties.Settings.Default.CH7Memo = textBoxCH7Memo.Text;
            Properties.Settings.Default.CH8Memo = textBoxCH8Memo.Text;
            Properties.Settings.Default.CH9Memo = textBoxCH9Memo.Text;
            Properties.Settings.Default.CH10Memo = textBoxCH10Memo.Text;
            Properties.Settings.Default.CH11Memo = textBoxCH11Memo.Text;
            Properties.Settings.Default.CH12Memo = textBoxCH12Memo.Text;
            Properties.Settings.Default.CH13Memo = textBoxCH13Memo.Text;

            Properties.Settings.Default.Save();
        }

        // シリアルポート接続
        private void btnConnect_Click(object sender, EventArgs e)
        {
            ComConnect();
            if (serialPort1.IsOpen)
            {
                serialPort1.Write("I1" + "\r");
            }
        }

        // シリアルポート切断
        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen)
            {
                ComDisConnect();
            }
        }

        // 以下、FT8受信バンドコマンド
        private void btnFT8_160m_Click(object sender, EventArgs e)
        {
            SetFT8Rcv(frqFT8_160m);
        }

        private void btnFT8_80m_Click(object sender, EventArgs e)
        {
            SetFT8Rcv(frqFT8_80m);
        }

        private void btnFT8_40m_Click(object sender, EventArgs e)
        {
            SetFT8Rcv(frqFT8_40m);
        }

        private void btnFT8_30m_Click(object sender, EventArgs e)
        {
            SetFT8Rcv(frqFT8_30m);
        }

        private void btnFT8_20m_Click(object sender, EventArgs e)
        {
            SetFT8Rcv(frqFT8_20m);
        }

        private void btnFT8_17m_Click(object sender, EventArgs e)
        {
            SetFT8Rcv(frqFT8_17m);
        }

        private void btnFT8_15m_Click(object sender, EventArgs e)
        {
            SetFT8Rcv(frqFT8_15m);
        }

        private void btnFT8_12m_Click(object sender, EventArgs e)
        {
            SetFT8Rcv(frqFT8_12m);
        }

        private void btnFT8_10m_Click(object sender, EventArgs e)
        {
            SetFT8Rcv(frqFT8_10m);
        }

        // マニュアルコマンド送信
        private void btnManualSend_Click(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen)
            {
                serialPort1.Write(textBoxCommand.Text + "\r");
                textBoxCommand.Clear();
            }
        }

        private void btnSaveUserSetup_Click(object sender, EventArgs e)
        {
            SaveUserData();
        }

        // 以下、ラジオ放送受信コマンド
        private void btnCH1_Click(object sender, EventArgs e)
        {
             SetRadioRcv(textBoxCH1Frq.Text);
        }

        private void btnCH2_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH2Frq.Text);
        }

        private void btnCH3_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH3Frq.Text);
        }

        private void btnCH4_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH4Frq.Text);
        }

        private void btnCH5_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH5Frq.Text);
        }

        private void btnCH6_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH6Frq.Text);
        }

        private void btnCH7_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH7Frq.Text);
        }

        private void btnCH8_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH8Frq.Text);
        }

        private void btnCH9_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH9Frq.Text);
        }

        private void btnCH10_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH10Frq.Text);
        }

        private void btnCH11_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH11Frq.Text);
        }

        private void btnCH12_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH12Frq.Text);
        }

        private void btnCH13_Click(object sender, EventArgs e)
        {
            SetRadioRcv(textBoxCH13Frq.Text);
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            DateTime utcNow = DateTime.UtcNow;
            labelUTCNow.Text = utcNow.ToString($"{utcNow:yyyy年MM月dd日(ddd)}{utcNow:  HH時mm分ss秒}");
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            SaveUserData();
            ComDisConnect();
            Close();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            SaveUserData();
            ComDisConnect();
        }

        // この一行は意味が分からない
        private delegate void Delegate_RcvDataToLabel(string data);

        // 半角 1 バイト、全角 2 バイトとして、指定された文字列のバイト数を返すメソッド
        public static int LenB(string stTarget)
        {
            return System.Text.Encoding.GetEncoding("Shift_JIS").GetByteCount(stTarget);
        }

        // 周波数を表示するメソッド
        private void ShowFrequency(string line)
        {
            string f1 = line.Substring(4, 2);
            string f2 = line.Substring(6, 3);
            string f3 = line.Substring(9, 2);
            string freq = f1 + "," + f2 + "." + f3 + "kHz";
            labelFrqNow.Text = freq;
        }
        // アッテネータを表示するメソッド
        private void ShowATT(string line)
        {            
            if (line.Substring(1,1)  == "1")
                {
                    labelATTNow.Text = "ON";
                    labelATT.ForeColor = System.Drawing.Color.Red;
                    labelATTNow.ForeColor = System.Drawing.Color.Red;
                }
            else
                {
                    labelATTNow.Text = "OFF";
                    labelATT.ForeColor = System.Drawing.Color.Black;
                    labelATTNow.ForeColor = System.Drawing.Color.Black;
                }
        }
        // バンド幅を表示するメソッド
        private void ShowBandWidth(string line)
        {
            string bw = line.Substring(2, 1);
            switch (bw)
            {
                case "0":
                    labelBandWidthNow.Text = "WIDE";
                    break;
                case "1":
                    labelBandWidthNow.Text = "INTER";
                    break;
                case "2":
                    labelBandWidthNow.Text = "NARR";
                    break;
                case "3":
                    labelBandWidthNow.Text = "AUX";
                    break;
                default:
                    labelBandWidthNow.Text = "error";
                    labelBandWidth.ForeColor = System.Drawing.Color.Red;
                    labelBandWidthNow.ForeColor = System.Drawing.Color.Red;
                    break;
            }
        }
        // モードを表示するメソッド
        private void ShowMode(string line)
        {
            string md = line.Substring(3, 1);
            switch (md)
            {
                case "0":
                    labelModeNow.Text = "RTTY";
                    break;
                case "1":
                    labelModeNow.Text = "CW";
                    break;
                case "2":
                    labelModeNow.Text = "USB";
                    break;
                case "3":
                    labelModeNow.Text = "LSB";
                    break;
                case "4":
                    labelModeNow.Text = "AM";
                    break;
                case "5":
                    labelModeNow.Text = "FM";
                    break;
                case "6":
                    labelModeNow.Text = "FAX";
                    break;
                case "7":
                    labelModeNow.Text = "ECSS-USB";
                    break;
                case "8":
                    labelModeNow.Text = "ECSS-LSB";
                    break;
                default:
                    labelModeNow.Text = "error";
                    labelMode.ForeColor = System.Drawing.Color.Red;
                    labelModeNow.ForeColor = System.Drawing.Color.Red;
                    break;
            }
        }
        // AGCを表示するメソッド
        private void ShowAGC(string line)
        {
            string agc = line.Substring(12, 1);
            switch (agc)
            {
                case "0":
                    labelAGCNow.Text = "SLOW";
                    break;
                case "1":
                    labelAGCNow.Text = "FAST";
                    break;
                case "2":
                    labelAGCNow.Text = "OFF";
                    break;
                default:
                    labelAGCNow.Text = "error";
                    labelAGC.ForeColor = System.Drawing.Color.Red;
                    labelAGCNow.ForeColor = System.Drawing.Color.Red;
                    break;
            }
        }

        // NRD-535設定状態を表示するメソッド
        private void ShowRadioStatus(string line)
        {
            if(line.Substring(0,1) == "I")  // ヘッダー信号識別
            {
                ShowFrequency(line);        // 周波数表示
                ShowMode(line);             // モード表示
                ShowBandWidth(line);        // バンド幅表示
                ShowAGC(line);              // AGC表示
                ShowATT(line);              // ATT表示
            }
        }

        // 受信データをラベルに書き込むメソッド
        private void RcvDataToLabel( string data)
        {
            StringReader sr;
            string line;

            if( data != null)
            {
                labelConnectStatus.Text = "NRD-535 接続中";
                labelConnectStatus.ForeColor = System.Drawing.Color.Blue;

                sr = new StringReader(data);
                while((line = sr.ReadLine()) != null)
                {
                    if(LenB(line) == 13)
                    {
                        labelDeBug.Text = line;
                        ShowRadioStatus(line);
                    }
                    
                }
            }
        }

        // データを受信してラベルに書き込む
        private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // シリアルポートをオープンしていない場合、処理を行わない
            if (serialPort1.IsOpen == false)
            {
                return;
            }
            try
            {
                // 受信データを読み込む
                string data = serialPort1.ReadExisting();

                // 受信したデータをラベルに書き込む
                Invoke(new Delegate_RcvDataToLabel(RcvDataToLabel), new Object[] { data });
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        
            
        }
    }
}
